library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity CGCNT is
	port (
		CGSEL		: in  std_logic;
		M1_N		: in  std_logic;
		IORQ_N		: in  std_logic;
		MREQ_N		: in  std_logic;
		WR_N		: in  std_logic;
		RD_N		: in  std_logic;
		RAS2_N		: in  std_logic;
		EXCAS_N		: in  std_logic;
		DRD2_N		: in  std_logic;
		BUFADD		: in  std_logic_vector(12 downto 0);
		BUFDATA		: in  std_logic_vector(7 downto 0);
		CG0ENB		: in  std_logic;
		CG0WR		: in  std_logic;
		CG1ENB		: in  std_logic;
		CG1WR		: in  std_logic;
		PCM0ENB		: in  std_logic;
		PCM0WR		: in  std_logic;
		PCM1ENB		: in  std_logic;
		PCM1WR		: in  std_logic;
		CGFRAME		: in  std_logic_vector(3 downto 0);
		PCMFRAME	: in  std_logic_vector(2 downto 0);
		CGFILEEND	: in  std_logic;
		PCMFILEEND	: in  std_logic;
		CTRLRST		: in  std_logic;
		SW			: in  std_logic_vector(1 downto 0);
		RSTN		: in  std_logic;
		CLK			: in  std_logic;
		Q			: out std_logic_vector(7 downto 0);
		CG0EMPTY	: out std_logic;
		CG1EMPTY	: out std_logic;
		PCM0EMPTY	: out std_logic;
		PCM1EMPTY	: out std_logic;
		MONOUT		: out std_logic_vector(19 downto 0)
	);
end CGCNT;

architecture RTL of CGCNT is

	component PCMROM is
		port (
			address		: in  std_logic_vector(7 downto 0);
			clock		: in  std_logic;
			q			: out std_logic_vector(11 downto 0)
		);
	end component;

	component SPRAM_8192W8B is
		port (
			address		: in  std_logic_vector(12 downto 0);
			data		: in  std_logic_vector(7 downto 0);
			wren		: in  std_logic;
			aclr		: in  std_logic;
			clock		: in  std_logic;
			q			: out std_logic_vector(7 downto 0)
		);
	end component;

	component SPRAM_1024W8B is
		port (
			address		: in  std_logic_vector(9 downto 0);
			data		: in  std_logic_vector(7 downto 0);
			wren		: in  std_logic;
			aclr		: in  std_logic;
			clock		: in  std_logic;
			q			: out std_logic_vector(7 downto 0)
		);
	end component;

	signal aclr			: std_logic;

	signal cgram_add_0	: std_logic_vector(12 downto 0);
	signal cgram_add_1	: std_logic_vector(12 downto 0);
	signal cgram_di		: std_logic_vector(7 downto 0);
	signal cgram_wr_0	: std_logic;
	signal cgram_wr_1	: std_logic;
	signal cgram_do_0	: std_logic_vector(7 downto 0);
	signal cgram_do_1	: std_logic_vector(7 downto 0);
	signal cgram_do		: std_logic_vector(7 downto 0);

	signal pcmram_add_0	: std_logic_vector(9 downto 0);
	signal pcmram_add_1	: std_logic_vector(9 downto 0);
	signal pcmram_di	: std_logic_vector(7 downto 0);
	signal pcmram_wr_0	: std_logic;
	signal pcmram_wr_1	: std_logic;
	signal pcmram_do_0	: std_logic_vector(7 downto 0);
	signal pcmram_do_1	: std_logic_vector(7 downto 0);
	signal pcmram_do	: std_logic_vector(7 downto 0);

	signal pcmrom_do	: std_logic_vector(11 downto 0);

	signal dispcnt		: std_logic_vector(7 downto 0);
	signal dispdet		: std_logic;
	signal dispdet_f1	: std_logic;
	signal dispdet_f	: std_logic;
	signal dvlicnt		: std_logic_vector(9 downto 0);
	signal dvlidet		: std_logic;
	signal dvlidet_f1	: std_logic;
	signal dvlidet_f2	: std_logic;
	signal dvlidet_r	: std_logic;
	signal hdspcnt		: std_logic_vector(7 downto 0);
	signal hdsppls		: std_logic;
	signal dplncnt		: std_logic_vector(11 downto 0);
	signal dplnpls		: std_logic;
	signal hlinecnt		: std_logic_vector(8 downto 0);
	signal hlinepls		: std_logic;

	signal cgsel_f1		: std_logic;
	signal cgsel_f2		: std_logic;
	signal cgsel_f3		: std_logic;
	signal genenb		: std_logic;
	signal genenb_f1	: std_logic;

	signal crtoninstmg	: std_logic;
	signal crtofinstmg	: std_logic;
	signal pcminstmg	: std_logic;
	signal pcmstart		: std_logic;

	signal mrd			: std_logic;
	signal mrd_f1		: std_logic;
	signal rdpls		: std_logic;
	signal rdpls_f1		: std_logic;

	signal progend		: std_logic;
	signal pcmins_pre	: std_logic;
	signal pcmins		: std_logic;
	signal pcmins_f1	: std_logic;
	signal pcmins_f		: std_logic;
	signal crtonins_pre	: std_logic;
	signal crtonins		: std_logic;
	signal crtofins_pre	: std_logic;
	signal crtofins		: std_logic;
	signal nopins		: std_logic;
	signal jmpins		: std_logic;
	signal addins		: std_logic_vector(3 downto 0);
	signal addlow		: std_logic_vector(1 downto 0);
	signal addhigh		: std_logic_vector(11 downto 0);
	signal addatt		: std_logic;
	signal addpage		: std_logic;

	signal addwd		: std_logic;
	signal cgdat		: std_logic_vector(15 downto 0);
	signal cgdat_lt		: std_logic_vector(15 downto 0);
	signal cgramcnt		: std_logic_vector(3 downto 0);
	signal cgfrmcnt		: std_logic_vector(3 downto 0);
	signal cgfrmend		: std_logic;
	signal cgfrmendins	: std_logic;

	signal datao		: std_logic_vector(7 downto 0);

	signal addpcm		: std_logic_vector(9 downto 0);
	signal addpcmf		: std_logic_vector(9 downto 0);
	signal pcmbank		: std_logic;
	signal pcmcnt		: std_logic_vector(2 downto 0);

	signal cg0empty_i	: std_logic;
	signal cg1empty_i	: std_logic;
	signal pcm0empty_i	: std_logic;
	signal pcm1empty_i	: std_logic;

	signal addhigh_0	: std_logic_vector(11 downto 0);
	signal addhigh_1	: std_logic_vector(11 downto 0);
	signal addhigh_2	: std_logic_vector(11 downto 0);
	signal addhigh_3	: std_logic_vector(11 downto 0);

begin

	U_SPRAM_CG0 : SPRAM_8192W8B
	port map (
		address		=> cgram_add_0,
		data		=> cgram_di,
		wren		=> cgram_wr_0,
		aclr		=> aclr,
		clock		=> CLK,
		q			=> cgram_do_0
	);

	U_SPRAM_CG1 : SPRAM_8192W8B
	port map (
		address		=> cgram_add_1,
		data		=> cgram_di,
		wren		=> cgram_wr_1,
		aclr		=> aclr,
		clock		=> CLK,
		q			=> cgram_do_1
	);

	U_SPRAM_PCM0 : SPRAM_1024W8B
	port map (
		address		=> pcmram_add_0,
		data		=> pcmram_di,
		wren		=> pcmram_wr_0,
		aclr		=> aclr,
		clock		=> CLK,
		q			=> pcmram_do_0
	);

	U_SPRAM_PCM1 : SPRAM_1024W8B
	port map (
		address		=> pcmram_add_1,
		data		=> pcmram_di,
		wren		=> pcmram_wr_1,
		aclr		=> aclr,
		clock		=> CLK,
		q			=> pcmram_do_1
	);

	U_PCMROM : PCMROM
	port map (
		address		=> pcmram_do,
		clock		=> CLK,
		q			=> pcmrom_do
	);

	aclr <= not RSTN;

	cgram_add_0  <= BUFADD when (CG0ENB  = '1') else addhigh & addwd;
	cgram_add_1  <= BUFADD when (CG1ENB  = '1') else addhigh & addwd;
	pcmram_add_0 <= BUFADD(9 downto 0) when (PCM0ENB = '1') else addpcm;
	pcmram_add_1 <= BUFADD(9 downto 0) when (PCM1ENB = '1') else addpcm;
	cgram_wr_0   <= CG0WR  when (CG0ENB  = '1') else '0';
	cgram_wr_1   <= CG1WR  when (CG1ENB  = '1') else '0';
	pcmram_wr_0  <= PCM0WR when (PCM0ENB = '1') else '0';
	pcmram_wr_1  <= PCM1WR when (PCM1ENB = '1') else '0';
	cgram_di     <= BUFDATA;
	pcmram_di    <= BUFDATA;
	cgram_do     <=	cgram_do_0  when (addatt  = '0' and cg0empty_i = '0') else
					cgram_do_1  when (addatt  = '1' and cg1empty_i = '0') else
					X"00";

	pcmram_do    <=	pcmram_do_0 when (pcmbank = '0' and pcm0empty_i = '0') else
					pcmram_do_1 when (pcmbank = '1' and pcm1empty_i = '0') else
					X"80";


-- detect display area
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dispcnt <= (others => '0');
			dispdet <= '0';
		elsif (CLK'event and CLK = '1') then
			if (M1_N = '0' or IORQ_N = '0' or
				MREQ_N = '0' or WR_N = '0' or RD_N = '0'
			) then
				dispcnt <= (others => '0');
				dispdet <= '0';
			elsif (dispcnt = X"FF") then
				dispdet <= '1';
			else
				dispcnt <= dispcnt + 1;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dispdet_f1 <= '0';
		elsif (CLK'event and CLK = '1') then
			dispdet_f1 <= dispdet;
		end if;
	end process;

	dispdet_f <= '1' when (dispdet = '0' and dispdet_f1 = '1') else '0';


-- detect vartical blanking interval
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dvlicnt <= (others => '0');
			dvlidet <= '0';
		elsif (CLK'event and CLK = '1') then
			if (dispdet = '1') then
				dvlicnt <= (others => '0');
				dvlidet <= '0';
			elsif (dvlicnt = "11" & X"FF") then
				dvlidet <= '1';
			else
				dvlicnt <= dvlicnt + 1;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dvlidet_f1 <= '0';
			dvlidet_f2 <= '0';
		elsif (CLK'event and CLK = '1') then
			dvlidet_f1 <= dvlidet;
			dvlidet_f2 <= dvlidet_f1;
		end if;
	end process;

	dvlidet_r <= '1' when (dvlidet_f1 = '1' and dvlidet_f2 = '0') else '0';


-- holizontal line count (display area)
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			hdspcnt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (dvlidet = '1') then
				hdspcnt <= (others => '0');
			elsif (hdsppls = '1') then
				hdspcnt <= hdspcnt + 1;
			end if;
		end if;
	end process;

	hdsppls <= '1' when (dvlidet = '0' and dispdet_f = '1') else '0';


-- holizontal line count (no display area)
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			dplncnt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (hdsppls = '1') then
				dplncnt <= (others => '0');
			elsif (dplnpls = '1') then
				dplncnt <= (others => '0');
			else
				dplncnt <= dplncnt + 1;
			end if;
		end if;
	end process;

	dplnpls <= '1' when (dvlidet = '1' and dplncnt = X"C70") else '0';


-- holizontal line count
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			hlinecnt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (dvlidet_r = '1') then
				hlinecnt <= "1" & X"05";	-- 261
			elsif (hlinepls = '1') then
				if (hlinecnt = 261) then
					hlinecnt <= (others => '0');
				else
					hlinecnt <= hlinecnt + 1;
				end if;
			end if;
		end if;
	end process;

	hlinepls <=	hdsppls or dplnpls;


-- program start timing generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cgsel_f1 <= '0';
			cgsel_f2 <= '0';
			cgsel_f3 <= '0';
		elsif (CLK'event and CLK = '1') then

			if (CGSEL = '0') then
				cgsel_f1 <= '0';
			elsif (dvlidet = '0') then
				cgsel_f1 <= '1';
			end if;

			if (CGSEL = '0') then
				cgsel_f2 <= '0';
			elsif (dvlidet = '1') then
				cgsel_f2 <= cgsel_f1;
			end if;

			cgsel_f3 <= cgsel_f2;

		end if;
	end process;

	genenb    <= cgsel_f2;
	genenb_f1 <= cgsel_f3;


-- crt on and page swap timing genarate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			crtoninstmg <= '0';
		elsif (CLK'event and CLK = '1') then
			if (genenb = '0') then
				crtoninstmg <= '0';
			elsif (dvlidet_r = '1') then
				crtoninstmg <= '1';
			elsif (crtonins_pre = '1') then
				crtoninstmg <= '0';
			end if;
		end if;
	end process;

-- crt off timing genarate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			crtofinstmg <= '0';
		elsif (CLK'event and CLK = '1') then
			if (genenb = '0') then
				crtofinstmg <= '0';
			elsif (PCMFRAME = "001" and hlinecnt = 108 and dplncnt = X"3FF") then
				crtofinstmg <= '1';
			elsif (PCMFRAME = "010" and hlinecnt = 110 and dplncnt = X"3FF") then
				crtofinstmg <= '1';
			elsif (crtofins_pre = '1') then
				crtofinstmg <= '0';
			end if;
		end if;
	end process;

-- PCM insert timing generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			pcminstmg <= '0';
		elsif (CLK'event and CLK = '1') then
			if (genenb = '0') then
				pcminstmg <= '0';
			elsif (pcmstart = '0') then
				pcminstmg <= '0';
			elsif (hlinepls = '1') then
				pcminstmg <= '1';
			elsif (pcmins_pre = '1') then
				pcminstmg <= '0';
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			pcmstart <= '0';
		elsif (CLK'event and CLK = '1') then
			if (genenb = '0') then
				pcmstart <= '0';
			elsif (dvlidet_r = '1') then
				pcmstart <= '1';
			end if;
		end if;
	end process;

-- PCM ram address generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			addpcm  <= (others => '0');
			addpcmf <= (others => '0');
			pcmcnt  <= (others => '0');
			pcmbank <= '0';
		elsif (CLK'event and CLK = '1') then

			if    (hlinecnt <  19) then
				addpcmf <= ("0" & hlinecnt) +  0;
			elsif (hlinecnt <  38) then
				addpcmf <= ("0" & hlinecnt) +  1;
			elsif (hlinecnt <  57) then
				addpcmf <= ("0" & hlinecnt) +  2;
			elsif (hlinecnt <  75) then
				addpcmf <= ("0" & hlinecnt) +  3;
			elsif (hlinecnt <  94) then
				addpcmf <= ("0" & hlinecnt) +  4;
			elsif (hlinecnt < 113) then
				addpcmf <= ("0" & hlinecnt) +  5;
			elsif (hlinecnt < 131) then
				addpcmf <= ("0" & hlinecnt) +  6;
			elsif (hlinecnt < 150) then
				addpcmf <= ("0" & hlinecnt) +  7;
			elsif (hlinecnt < 169) then
				addpcmf <= ("0" & hlinecnt) +  8;
			elsif (hlinecnt < 188) then
				addpcmf <= ("0" & hlinecnt) +  9;
			elsif (hlinecnt < 206) then
				addpcmf <= ("0" & hlinecnt) + 10;
			elsif (hlinecnt < 225) then
				addpcmf <= ("0" & hlinecnt) + 11;
			elsif (hlinecnt < 245) then
				addpcmf <= ("0" & hlinecnt) + 12;
			else
				addpcmf <= ("0" & hlinecnt) + 13;
			end if;

			addpcm <= (hlinecnt & "0") + addpcmf;


			if (PCMFRAME = "001") then
				pcmcnt <= "100";
			elsif (genenb = '0') then
				pcmcnt <= "000";
			elsif (pcmins_f = '1') then
				if (hlinecnt = 261) then
					pcmcnt <= "000";
				else
					if (pcmcnt = 3) then
						pcmcnt <= "000";
					else
						pcmcnt <= pcmcnt + 1;
					end if;
				end if;
			end if;

			if (genenb = '0') then
				pcmbank <= '0';
			elsif (hlinecnt = 261 and pcmins_f = '1') then
				pcmbank <= not pcmbank;
			end if;

		end if;
	end process;


-- read timing generate
	mrd <= '1' when (MREQ_N = '0' and RD_N = '0' and
					RAS2_N = '0' and EXCAS_N = '0' and DRD2_N = '0') else '0';

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			mrd_f1 <= '0';
		elsif (CLK'event and CLK = '1') then
			mrd_f1 <= mrd;
		end if;
	end process;

	rdpls <= '1' when (mrd = '0' and mrd_f1 = '1') else '0';

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			rdpls_f1 <= '0';
		elsif (CLK'event and CLK = '1') then
			rdpls_f1 <= rdpls;
		end if;
	end process;


-- address high parameter

	addhigh_0 <= X"F8C";
	addhigh_1 <= X"F8B";
	addhigh_2 <= X"3D3" when (PCMFRAME = "001") else X"3FB";
	addhigh_3 <= X"3D2" when (PCMFRAME = "001") else X"3FA";


-- read data generation control and CG ram address generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			progend      <= '0';
			crtonins_pre <= '0';
			crtonins     <= '0';
			crtofins_pre <= '0';
			crtofins     <= '0';
			pcmins_pre   <= '0';
			pcmins       <= '0';
			nopins       <= '0';
			jmpins       <= '0';
			addins       <= (others => '0');
			addlow       <= (others => '0');
			addhigh      <= (others => '0');
			addatt       <= '0';
			addpage      <= '0';
		elsif (CLK'event and CLK = '1') then
			if (rdpls = '1') then
				if (genenb = '0') then
					progend      <= '0';
					pcmins_pre   <= '0';
					pcmins       <= '0';
					crtonins_pre <= '0';
					crtonins     <= '0';
					crtofins_pre <= '0';
					crtofins     <= '0';
					nopins       <= '0';
					jmpins       <= '0';
					addins       <= (others => '0');
					addlow       <= (others => '0');
					addhigh      <= addhigh_0;
					addatt       <= '0';
					addpage      <= '0';
				elsif (progend  = '1') then
					if (addins = 5) then
						addins   <= "0000";
						progend  <= '0';
					else
						addins   <= addins + 1;
					end if;
				elsif (pcmins   = '1') then
					if (hlinecnt = 261) then
						jmpins <= '1';
					end if;
					if (
							(pcmcnt = 0 and addins = 2) or
							(pcmcnt = 1 and addins = 6) or
							(pcmcnt = 2 and addins = 2) or
							(pcmcnt = 3 and addins = 6) or
							(pcmcnt = 4 and addins = 3)
					) then
						addins   <= "0000";
						pcmins   <= '0';
					else
						addins   <= addins + 1;
					end if;
				elsif (crtonins = '1') then
					if (addins = 7) then
						addins   <= "0000";
						crtonins <= '0';
					else
						addins   <= addins + 1;
					end if;
				elsif (crtofins = '1') then
					if (addins = 3) then
						addins   <= "0000";
						crtofins <= '0';
					else
						addins   <= addins + 1;
					end if;
				elsif (jmpins = '1') then
					if (addins = 2) then
						addins   <= "0000";
						jmpins   <= '0';
					else
						addins   <= addins + 1;
					end if;
				elsif (nopins = '1' and (pcminstmg = '0' and pcmins_pre = '0')) then
					if (CGFILEEND = '1') then
						progend  <= '1';
						addins   <= "0000";
					end if;
				elsif (nopins = '1' and (pcminstmg = '1' or  pcmins_pre = '1')) then
					if (pcminstmg = '1') then
						pcmins_pre <= '1';
					else
						if (hlinecnt = 261) then
							nopins   <= '0';
						end if;
						pcmins_pre <= '0';
						pcmins <= '1';
						addins <= "0000";
					end if;
				elsif (addhigh = addhigh_3 and cgfrmendins = '1') then
					addhigh <= addhigh_0;
					addatt  <= '0';
					addpage <= not addpage;
				else
					case addlow is
						when "00" =>
							addlow <= "01";
						when "01" =>
							addlow <= "10";
							if    (pcminstmg = '1') then
								pcmins_pre <= '1';
							elsif (pcmins_pre = '1') then
								pcmins_pre <= '0';
								pcmins   <= '1';
								addins   <= "0000";
							elsif (crtoninstmg = '1') then
								crtonins_pre <= '1';
							elsif (crtonins_pre = '1') then
								crtonins_pre <= '0';
								crtonins <= '1';
								addins   <= "0000";
							elsif (crtofinstmg = '1') then
								crtofins_pre <= '1';
							elsif (crtofins_pre = '1') then
								crtofins_pre <= '0';
								crtofins <= '1';
								nopins   <= '1';
								addins   <= "0000";
							end if;
						when "10" =>
							addlow <= "00";
							if (addhigh = addhigh_2) then
								if (addatt = '0') then
									addhigh <= addhigh_0;
									addatt  <= '1';
								else
									addhigh <= addhigh - 1;
								end if;
							elsif (addhigh /= addhigh_3) then
								addhigh <= addhigh - 1;
							end if;
						when others =>
							null;
					end case;

				end if;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			pcmins_f1 <= '0';
		elsif (CLK'event and CLK = '1') then
			pcmins_f1 <= pcmins;
		end if;
	end process;

	pcmins_f <= '1' when (pcmins = '0' and pcmins_f1 = '1') else '0';

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cgramcnt <= "1111";
			addwd <= '0';
		elsif (CLK'event and CLK = '1') then
			if (rdpls_f1 = '1' and addlow = 2) then
				cgramcnt <= "0000";
				addwd <= '1';
				cgramcnt <= cgramcnt + 1;
			elsif (cgramcnt /= "1111") then
				cgramcnt <= cgramcnt + 1;
				if (cgramcnt = "0100") then
					addwd <= '0';
				end if;
			end if;
		end if;
	end process;

-- CG data generate
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cgdat <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if    (cgramcnt = "0011") then
				cgdat(15 downto 8) <= cgram_do;
			elsif (cgramcnt = "0111") then
				cgdat(7 downto 0) <= cgram_do;
			elsif (cgramcnt = "1000") then
				if (pcmins_pre = '1') then
					case pcmcnt is
						when "000"  => cgdat <= cgdat - 6;
						when "001"  => cgdat <= cgdat - 10;
						when "010"  => cgdat <= cgdat - 6;
						when "011"  => cgdat <= cgdat - 10;
						when others => cgdat <= cgdat - 7;
					end case;
				elsif (crtonins_pre = '1') then
					cgdat <= cgdat - 11;
				else
					cgdat <= cgdat - 3;
				end if;
			end if;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cgdat_lt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (rdpls = '1' and addlow = 1 and crtofins_pre = '1') then
				cgdat_lt <= cgdat;
			end if;
		end if;
	end process;

-- CG frame counter
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cgfrmcnt <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (genenb_f1 = '0') then
				cgfrmcnt <= (others => '0');
			elsif (dvlidet_r = '1') then
				if (cgfrmcnt = CGFRAME - 1) then
					cgfrmcnt <= (others => '0');
				else
					cgfrmcnt <= cgfrmcnt + 1;
				end if;
			end if;
		end if;
	end process;

	cgfrmend <= '1' when (dvlidet_r = '1' and cgfrmcnt = CGFRAME - 1) else '0';

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cgfrmendins <= '0';
		elsif (CLK'event and CLK = '1') then
			if (genenb = '0') then
				cgfrmendins <= '0';
			elsif (cgfrmend = '1') then
				cgfrmendins <= '1';
			elsif (addatt = '0') then
				cgfrmendins <= '0';
			end if;
		end if;
	end process;


-- output program to PC-6001
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			datao <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if (genenb = '0') then
				datao <= (others => '0');			-- NOP
			elsif (progend = '1') then
				case addins is
					when "0000" => datao <= X"C3";	-- CALL C000H
					when "0001" => datao <= X"00";	-- CALL C000H
					when "0010" => datao <= X"C0";	-- CALL C000H
					when "0011" => datao <= X"AF";	-- 0xC000:XOR A
					when "0100" => datao <= X"D3";	-- 0xC001:OUT (15H),A
					when "0101" => datao <= X"15";	-- 0xC002:OUT (15H),A
					when others => datao <= X"00";
				end case;
			elsif (pcmins  = '1') then
				if   (pcmcnt = 0 or pcmcnt = 2) then
					case addins is
						when "0000" => datao <= X"11";	-- LD  DE,XXXX
						when "0001" => datao <= "0000" & pcmrom_do(7 downto 4);	-- LD  DE,XXXX
						when "0010" => datao <= "0000" & pcmrom_do(3 downto 0);	-- LD  DE,XXXX
						when others => datao <= X"00";	-- NOP
					end case;
				elsif (pcmcnt = 1) then
					case addins is
						when "0000" => datao <= X"7C";	-- LD  A,H
						when "0001" => datao <= X"ED";	-- OUT (C),D
						when "0010" => datao <= X"51";	-- OUT (C),D
						when "0011" => datao <= X"D3";	-- OUT (0A0H),A
						when "0100" => datao <= X"A0";	-- OUT (0A0H),A
						when "0101" => datao <= X"ED";	-- OUT (C),E
						when "0110" => datao <= X"59";	-- OUT (C),E
						when others => datao <= X"00";	-- NOP
					end case;
				elsif (pcmcnt = 3) then
					case addins is
						when "0000" => datao <= X"7D";	-- LD  A,L
						when "0001" => datao <= X"ED";	-- OUT (C),E
						when "0010" => datao <= X"59";	-- OUT (C),E
						when "0011" => datao <= X"D3";	-- OUT (0A0H),A
						when "0100" => datao <= X"A0";	-- OUT (0A0H),A
						when "0101" => datao <= X"ED";	-- OUT (C),D
						when "0110" => datao <= X"51";	-- OUT (C),D
						when others => datao <= X"00";	-- NOP
					end case;
				else
					case addins is
						when "0000" => datao <= X"3E";	-- LD  A,XX
						when "0001" => datao <= "0000" & pcmrom_do(11 downto 8);	-- LD  A,XX
						when "0010" => datao <= X"D3";	-- OUT (0A1H),A
						when "0011" => datao <= X"A1";	-- OUT (0A1H),A
						when others => datao <= X"00";
					end case;
				end if;
			elsif (crtonins  = '1') then
				case addins is
					when "0000" => datao <= X"3E";	-- LD  A,02H
					when "0001" => datao <= X"02";	-- LD  A,02H
					when "0010" => datao <= X"D3";	-- OUT (093H),A
					when "0011" => datao <= X"93";	-- OUT (093H),A
					when "0100" => datao <= X"3E";	-- LD  A,0F4H/0F6H
					when "0101" => datao <= "111101" & addpage & "0";	-- LD  A,0F4H/0F6H
					when "0110" => datao <= X"D3";	-- OUT (0B1H),A
					when "0111" => datao <= X"B1";	-- OUT (0B1H),A
					when others => datao <= X"00";
				end case;
			elsif (crtofins  = '1') then
				case addins is
					when "0000" => datao <= X"3E";	-- LD  A,03H
					when "0001" => datao <= X"03";	-- LD  A,03H
					when "0010" => datao <= X"D3";	-- OUT (093H),A
					when "0011" => datao <= X"93";	-- OUT (093H),A
					when others => datao <= X"00";
				end case;
			elsif (jmpins    = '1') then
				case addins is
					when "0000" => datao <= X"C3";					-- JP XXXX
					when "0001" => datao <= cgdat_lt( 7 downto 0);	-- JP XXXX
					when "0010" => datao <= cgdat_lt(15 downto 8);	-- JP XXXX
					when others => datao <= X"00";
				end case;
			elsif (nopins = '1') then
				datao <= X"00";						-- NOP
			elsif (addhigh = addhigh_3) then
				datao <= X"00";						-- NOP
			elsif (addhigh = addhigh_0) then
				case addlow is
					when "00"   => datao <= X"00";	-- no use / CALL 0000H
					when "01"   => datao <= X"00";	-- NOP / CALL 0000H
					when "10"   => datao <= X"31";	-- LD SP,7F/5F/3F/1F18H
					when others => datao <= X"00";
				end case;
			elsif (addhigh = addhigh_1) then
				case addlow is
					when "00"   => datao <= X"18";	-- LD SP,7F/5F/3F/1F18H
					when "01"   => datao <= "0" & addpage & addatt & "11111";	-- LD SP,7F/5F/3F/1F18H
					when "10"   => datao <= X"C3";	-- CALL XXXX
					when others => datao <= X"00";
				end case;
			else
				case addlow is
					when "00"   => datao <= cgdat( 7 downto 0);	-- CALL XXXX
					when "01"   => datao <= cgdat(15 downto 8);	-- CALL XXXX
					when "10"   => datao <= X"CD";					-- CALL XXXX
					when others => datao <= X"00";
				end case;
			end if;
		end if;
	end process;


-- CG FIFO empty flag
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			cg0empty_i <= '1';
			cg1empty_i <= '1';
		elsif (CLK'event and CLK = '1') then

			if (CTRLRST = '1') then
				cg0empty_i <= '1';
			elsif (rdpls = '1' and addlow = 2 and addhigh = addhigh_2 and addatt = '0') then
				cg0empty_i <= '1';
			elsif (CG0WR = '1' and CG0ENB = '1') then
				cg0empty_i <= '0';
			end if;

			if (CTRLRST = '1') then
				cg0empty_i <= '1';
			elsif (rdpls = '1' and addlow = 2 and addhigh = addhigh_2 and addatt = '1') then
				cg1empty_i <= '1';
			elsif (CG1WR = '1' and CG1ENB = '1') then
				cg1empty_i <= '0';
			end if;

		end if;
	end process;


-- PCM FIFO empty flag
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			pcm0empty_i <= '1';
			pcm1empty_i <= '1';
		elsif (CLK'event and CLK = '1') then

			if (CTRLRST = '1') then
				pcm0empty_i <= '1';
			elsif (hlinecnt = 261 and pcmins_f = '1' and pcmbank = '0') then
				pcm0empty_i <= '1';
			elsif (PCM0WR = '1' and PCM0ENB = '1') then
				pcm0empty_i <= '0';
			end if;

			if (CTRLRST = '1') then
				pcm1empty_i <= '1';
			elsif (hlinecnt = 261 and pcmins_f = '1' and pcmbank = '1') then
				pcm1empty_i <= '1';
			elsif (PCM1WR = '1' and PCM1ENB = '1') then
				pcm1empty_i <= '0';
			end if;

		end if;
	end process;


	Q <= datao;

	CG0EMPTY  <= cg0empty_i;
	CG1EMPTY  <= cg1empty_i;
	PCM0EMPTY <= pcm0empty_i;
	PCM1EMPTY <= pcm1empty_i;


	MONOUT(0) <= pcminstmg;
	MONOUT(1) <= pcmins_pre;
	MONOUT(2) <= pcmins;
	MONOUT(3) <= dispdet;
	MONOUT(4) <= dvlidet;
	MONOUT(5) <= hdsppls;
	MONOUT(6) <= dplnpls;
	MONOUT(7) <= hlinepls;
	MONOUT(8) <= genenb;

	MONOUT(19 downto 9) <= (others => '0');

--	MONOUT(19 downto 8) <= addhigh;

--	MONOUT(15 downto 8) <= datao;

--	MONOUT(17 downto 8) <= addpcm;
--	MONOUT(18) <= pcmbank;

--	MONOUT(16) <= crtonins;
--	MONOUT(17) <= crtofins;
--	MONOUT(18) <= nopins;

end RTL;
