--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity CLKGEN is
	port (
		CLK50M		: in  std_logic;
		RSTN		: in  std_logic;
		CLK12MOUT	: out std_logic;
		CLK50MOUT	: out std_logic;
		CLK1SOUT	: out std_logic;
		LOCK_PLL	: out std_logic;
		CPU_RSTN	: out std_logic
	);
end CLKGEN;

architecture RTL of CLKGEN is

	component PLL12M100M is
		port (
			INCLK0	: in  std_logic;
			C0		: out std_logic;
			C1		: out std_logic;
			LOCKED	: out std_logic
		);
	end component;

	signal clk12m		: std_logic;
	signal clk50m_o		: std_logic;

	signal lock_12m		: std_logic;

	signal clk1scntlow	: std_logic_vector(15 downto 0);
	signal clk1scnthigh	: std_logic_vector(8 downto 0);
	signal clk1sout_i	: std_logic;

	signal cpurstn_f1	: std_logic;
	signal cpurstn_f2	: std_logic;
	signal cpurstn_f3	: std_logic;
	signal cpurstn_f4	: std_logic;

begin

	U_PLL12M100M : PLL12M100M
	port map (
		INCLK0	=> CLK50M,
		C0		=> clk12m,
		C1		=> clk50m_o,
		LOCKED	=> lock_12m
	);

	process (clk50m_o,RSTN)
	begin
		if (RSTN = '0') then
			clk1scntlow <= (others => '0');
		elsif (clk50m_o'event and clk50m_o = '1') then
			if (clk1scntlow = 49999) then
				clk1scntlow <= (others => '0');
			else
				clk1scntlow <= clk1scntlow + 1;
			end if;
		end if;
	end process;

	process (clk50m_o,RSTN)
	begin
		if (RSTN = '0') then
			clk1scnthigh <= (others => '0');
			clk1sout_i <= '0';
		elsif (clk50m_o'event and clk50m_o = '1') then
			if (clk1scntlow = 49999) then
				if (clk1scnthigh = 499) then
					clk1scnthigh <= (others => '0');
					clk1sout_i <= not clk1sout_i;
				else
					clk1scnthigh <= clk1scnthigh + 1;
				end if;
			end if;
		end if;
	end process;

	process (clk12m,RSTN)
	begin
		if (RSTN = '0') then
			cpurstn_f1 <= '0';
			cpurstn_f2 <= '0';
			cpurstn_f3 <= '0';
			cpurstn_f4 <= '0';
		elsif (clk12m'event and clk12m = '0') then
			cpurstn_f1 <= lock_12m;
			cpurstn_f2 <= cpurstn_f1;
			cpurstn_f3 <= cpurstn_f2;
			cpurstn_f4 <= cpurstn_f3;
		end if;
	end process;


	CLK50MOUT  <= clk50m_o;
	CLK12MOUT  <= clk12m;
	CLK1SOUT   <= clk1sout_i;

	LOCK_PLL   <= lock_12m and RSTN;
	CPU_RSTN   <= cpurstn_f4;

end RTL;
