--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity CTRLROM_WRAP is
	port (
		ADDRESS	: in  std_logic_vector(11 downto 0);
		DATA	: in  std_logic_vector(7 downto 0);
		RDN		: in  std_logic;
		WRN		: in  std_logic;
		CLK		: in  std_logic;
		RSTN	: in  std_logic;
		Q		: out std_logic_vector(7 downto 0)
	);
end CTRLROM_WRAP;

architecture RTL of CTRLROM_WRAP is

	component CTRLROM is
		port (
			aclr	: in  std_logic;
			address	: in  std_logic_vector(11 downto 0);
			clock	: in  std_logic;
			data	: in  std_logic_vector(7 downto 0);
			wren	: in  std_logic;
			q		: out std_logic_vector(7 downto 0)
		);
	end component;

	signal add		: std_logic_vector(11 downto 0);
	signal q_i		: std_logic_vector(7 downto 0);

	signal add_f1	: std_logic_vector(11 downto 0);
	signal rdn_f1	: std_logic;
	signal wrn_f1	: std_logic;

	signal wren		: std_logic;
	signal aclr		: std_logic;

begin

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			rdn_f1 <= '1';
		elsif (CLK'event and CLK = '1') then
			rdn_f1 <= RDN;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			wrn_f1 <= '1';
		elsif (CLK'event and CLK = '1') then
			wrn_f1 <= WRN;
		end if;
	end process;

	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			add_f1 <= (others => '0');
		elsif (CLK'event and CLK = '1') then
			if    (WRN = '0' and wrn_f1 = '1') then
				add_f1 <= ADDRESS;
			elsif (RDN = '0' and rdn_f1 = '1') then
				add_f1 <= ADDRESS;
			end if;
		end if;
	end process;

	add  <= ADDRESS when (WRN = '0' or RDN = '0') else add_f1;
	aclr <= not RSTN;
	wren <= '1' when (WRN = '0' and wrn_f1 = '1') else '0';

	U_CTRLROM : CTRLROM
	port map (
		aclr	=> aclr,
		address	=> add,
		clock	=> CLK,
		data    => DATA,
		wren    => wren,
		q		=> q_i
	);

	Q <= q_i;

end RTL;
