-- MOVIE for PC-6001mk2
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity P6MOVIE_DE0 is
	port (
		DRAM_D			: inout std_logic_vector(15 downto 0);	-- SDRAM data bus
		DRAM_A			: out   std_logic_vector(12 downto 0);	-- SDRAM address bus
		DRAM_CLK		: out   std_logic;						-- SDRAM clock output
		DRAM_CKE		: out   std_logic;						-- SDRAM clock enable
		DRAM_LDQM		: out   std_logic;						-- SDRAM LowerByte Data Mask
		DRAM_UDQM		: out   std_logic;						-- SDRAM UpperByte Data Mask
		DRAM_WE_N		: out   std_logic;						-- SDRAM write Enable
		DRAM_CAS_N		: out   std_logic;						-- SDRAM CAS
		DRAM_RAS_N		: out   std_logic;						-- SDRAM RAS
		DRAM_CS_N		: out   std_logic;						-- SDRAM chip select
		DRAM_BA_1		: out   std_logic;						-- SDRAM Bank #1
		DRAM_BA_0		: out   std_logic;						-- SDRAM Bank #0
		FLASH_D15_AM1	: inout std_logic;						-- FLASH data bus bit15 or adrress-1
		FLASH_D			: inout std_logic_vector(14 downto 0);	-- FLASH data bus
		FLASH_A			: out   std_logic_vector(21 downto 0);	-- FLASH adrress bus
		FLASH_WE_N		: out   std_logic;						-- FLASH write enable
		FLASH_RESET_N	: out   std_logic;						-- FLASH reset
		FLASH_WP_N		: out   std_logic;						-- FLASH write protect
		FLASH_RY		: in    std_logic;						-- FLASH ready
		FLASH_CE_N		: out   std_logic;						-- FLASH chip enable
		FLASH_OE_N		: out   std_logic;						-- FLASH output enable
		FLASH_BYTE_N	: out   std_logic;						-- FLASH byte mode
		VGA_R			: out   std_logic_vector(3 downto 0);	-- VGA red data
		VGA_G			: out   std_logic_vector(3 downto 0);	-- VGA green data
		VGA_B			: out   std_logic_vector(3 downto 0);	-- VGA blue data
		VGA_HS			: out   std_logic;						-- VGA H_SYNC
		VGA_VS			: out   std_logic;						-- VGA V_SYNC
		HEX3_D			: out   std_logic_vector(6 downto 0);	-- 7segment #3
		HEX3_DP			: out   std_logic;						-- 7segment #3 DP
		HEX2_D			: out   std_logic_vector(6 downto 0);	-- 7segment #2
		HEX2_DP			: out   std_logic;						-- 7segment #2 DP
		HEX1_D			: out   std_logic_vector(6 downto 0);	-- 7segment #1
		HEX1_DP			: out   std_logic;						-- 7segment #1 DP
		HEX0_D			: out   std_logic_vector(6 downto 0);	-- 7segment #0
		HEX0_DP			: out   std_logic;						-- 7segment #0 DP
		LEDG			: out   std_logic_vector(9 downto 0);	-- LED
		LCD_D			: inout std_logic_vector(7 downto 0);	-- LCD data bus
		LCD_BLON		: out   std_logic;						-- LCD back light on
		LCD_RS			: out   std_logic;						-- LCD command/data select
		LCD_RW			: out   std_logic;						-- LCD read/write
		LCD_EN			: out   std_logic;						-- LCD enable
		CLK50M1			: in    std_logic;						-- clock 50MHz input #1
		CLK50M0			: in    std_logic;						-- clock 50MHz input #0
		UART_RXD		: in    std_logic;						-- UART Rx
		UART_RTS		: in    std_logic;						-- UART CTS(!!)
		UART_TXD		: out   std_logic;						-- UART Tx
		UART_CTS		: out   std_logic;						-- UART RTS(!!)
		PS2_KBDAT		: inout std_logic;						-- PS2 keyboard data
		PS2_KBCLK		: inout std_logic;						-- PS2 keyboard clock
		PS2_MSDAT		: inout std_logic;						-- PS2 mouse data
		PS2_MSCLK		: inout std_logic;						-- PS2 mouse clock
		BUTTON			: in    std_logic_vector(2 downto 0);	-- push button
		SW				: in    std_logic_vector(9 downto 0);	-- DIPSW
		GPIO0_D_00		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_01		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_02		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_03		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_04		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_05		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_06		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_07		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_08		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_09		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_10		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_11		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_12		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_13		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_14		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_15		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_16		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_17		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_18		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_19		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_20		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_21		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_22		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_23		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_24		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_25		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_26		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_27		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_28		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_29		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_30		: inout std_logic;						-- GPIO #0 data
		GPIO0_D_31		: inout std_logic;						-- GPIO #0 data
		GPIO0_CLKIN_0	: in    std_logic;						-- GPIO #0 clock input
		GPIO0_CLKIN_1	: in    std_logic;						-- GPIO #0 clock input
		GPIO0_CLKOUT_0	: out   std_logic;						-- GPIO #0 clock output
		GPIO0_CLKOUT_1	: out   std_logic;						-- GPIO #0 clock output
		GPIO1_D_00		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_01		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_02		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_03		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_04		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_05		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_06		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_07		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_08		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_09		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_10		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_11		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_12		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_13		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_14		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_15		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_16		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_17		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_18		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_19		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_20		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_21		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_22		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_23		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_24		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_25		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_26		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_27		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_28		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_29		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_30		: inout std_logic;						-- GPIO #1 data
		GPIO1_D_31		: inout std_logic;						-- GPIO #1 data
		GPIO1_CLKIN_0	: in    std_logic;						-- GPIO #1 clock input
		GPIO1_CLKIN_1	: in    std_logic;						-- GPIO #1 clock input
		GPIO1_CLKOUT_0	: out   std_logic;						-- GPIO #1 clock output
		GPIO1_CLKOUT_1	: out   std_logic;						-- GPIO #1 clock output
		SD_DAT			: inout std_logic_vector(3 downto 0);	-- SD card data
		SD_CMD			: inout std_logic;						-- SD card command
		SD_CLK			: out   std_logic;						-- SD card clock output
		SD_WP_N			: in    std_logic						-- SD card write protect
	);
end P6MOVIE_DE0;

architecture RTL of P6MOVIE_DE0 is

	component P6MOVIE is
		port (
			CLK50M			: in    std_logic;						-- clock 50MHz input #1
			LED				: out   std_logic_vector(7 downto 0);	-- LED
			BUTTON			: in    std_logic_vector(1 downto 0);	-- push button
			SW				: in    std_logic_vector(3 downto 0);	-- DIPSW
			DRAM_D			: inout std_logic_vector(15 downto 0);	-- SDRAM data bus
			DRAM_A			: out   std_logic_vector(12 downto 0);	-- SDRAM address bus
			DRAM_CLK		: out   std_logic;						-- SDRAM clock output
			DRAM_CKE		: out   std_logic;						-- SDRAM clock enable
			DRAM_LDQM		: out   std_logic;						-- SDRAM LowerByte Data Mask
			DRAM_UDQM		: out   std_logic;						-- SDRAM UpperByte Data Mask
			DRAM_WE_N		: out   std_logic;						-- SDRAM write Enable
			DRAM_CAS_N		: out   std_logic;						-- SDRAM CAS
			DRAM_RAS_N		: out   std_logic;						-- SDRAM RAS
			DRAM_CS_N		: out   std_logic;						-- SDRAM chip select
			DRAM_BA			: out   std_logic_vector(1 downto 0);	-- SDRAM Bank
			I2C_SCLK		: out   std_logic;						-- I2C clock output
			I2C_SDAT		: inout std_logic;						-- I2C data
			G_SENSOR_CS_N	: out   std_logic;						-- G-SENSOR CS output
			G_SENSOR_INT	: in    std_logic;						-- G-SENSOR data input
			ADC_CS_N		: out   std_logic;						-- A/D CS output
			ADC_SADDR		: out   std_logic;						-- A/D address output
			ADC_SCLK		: out   std_logic;						-- A/D clock output
			ADC_SDAT		: in    std_logic;						-- A/D data input
			GPIO0_D_00		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_01		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_02		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_03		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_04		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_05		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_06		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_07		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_08		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_09		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_10		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_11		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_12		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_13		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_14		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_15		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_16		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_17		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_18		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_19		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_20		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_21		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_22		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_23		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_24		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_25		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_26		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_27		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_28		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_29		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_30		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_31		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_32		: inout std_logic;						-- GPIO #0 data
			GPIO0_D_33		: inout std_logic;						-- GPIO #0 data
			GPIO0_CLKIN_0	: in    std_logic;						-- GPIO #0 clock input
			GPIO0_CLKIN_1	: in    std_logic;						-- GPIO #0 clock input
			GPIO1_D_00		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_01		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_02		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_03		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_04		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_05		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_06		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_07		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_08		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_09		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_10		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_11		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_12		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_13		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_14		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_15		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_16		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_17		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_18		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_19		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_20		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_21		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_22		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_23		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_24		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_25		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_26		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_27		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_28		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_29		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_30		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_31		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_32		: inout std_logic;						-- GPIO #1 data
			GPIO1_D_33		: inout std_logic;						-- GPIO #1 data
			GPIO1_CLKIN_0	: in    std_logic;						-- GPIO #1 clock input
			GPIO1_CLKIN_1	: in    std_logic;						-- GPIO #1 clock input
			GPIO2_D_00		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_01		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_02		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_03		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_04		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_05		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_06		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_07		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_08		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_09		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_10		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_11		: inout std_logic;						-- GPIO #2 data
			GPIO2_D_12		: inout std_logic;						-- GPIO #2 data
			GPIO2_CLKIN_0	: in    std_logic;						-- GPIO #2 clock input
			GPIO2_CLKIN_1	: in    std_logic;						-- GPIO #2 clock input
			GPIO2_CLKIN_2	: in    std_logic						-- GPIO #2 clock input
		);
	end component;

	signal button_nano		: std_logic_vector(1 downto 0);
	signal sw_nano			: std_logic_vector(3 downto 0);
	signal dram_ba_nano		: std_logic_vector(1 downto 0);
	signal led_nano			: std_logic_vector(7 downto 0);
	signal i2c_sdat_nano	: std_logic;

	signal gpio0_clkin_0_nano		: std_logic;
	signal gpio0_clkout_0_nano		: std_logic;
	signal gpio0_clkout_1_nano		: std_logic;
	signal gpio1_clkout_0_nano		: std_logic;
	signal gpio1_clkout_1_nano		: std_logic;

	signal sd_wp_nano		: std_logic;
	signal sd_clk_nano		: std_logic;

	signal gpio2_d_nano		: std_logic_vector(12 downto 0);

begin

	U_P6MOVIE : P6MOVIE
	port map (
		CLK50M			=> CLK50M0,
		LED				=> led_nano,
		BUTTON			=> button_nano,
		SW				=> sw_nano,
		DRAM_D			=> DRAM_D,
		DRAM_A			=> DRAM_A,
		DRAM_CLK		=> DRAM_CLK,
		DRAM_CKE		=> DRAM_CKE,
		DRAM_LDQM		=> DRAM_LDQM,
		DRAM_UDQM		=> DRAM_UDQM,
		DRAM_WE_N		=> DRAM_WE_N,
		DRAM_CAS_N		=> DRAM_CAS_N,
		DRAM_RAS_N		=> DRAM_RAS_N,
		DRAM_CS_N		=> DRAM_CS_N,
		DRAM_BA			=> dram_ba_nano,
		I2C_SCLK		=> open,
		I2C_SDAT		=> i2c_sdat_nano,
		G_SENSOR_CS_N	=> open,
		G_SENSOR_INT	=> '0',
		ADC_CS_N		=> open,
		ADC_SADDR		=> open,
		ADC_SCLK		=> open,
		ADC_SDAT		=> '0',
		GPIO0_D_00		=> GPIO0_D_00,
		GPIO0_D_01		=> GPIO0_D_01,
		GPIO0_D_02		=> GPIO0_D_02,
		GPIO0_D_03		=> GPIO0_D_03,
		GPIO0_D_04		=> GPIO0_D_04,
		GPIO0_D_05		=> GPIO0_D_05,
		GPIO0_D_06		=> GPIO0_D_06,
		GPIO0_D_07		=> GPIO0_D_07,
		GPIO0_D_08		=> GPIO0_D_08,
		GPIO0_D_09		=> GPIO0_D_09,
		GPIO0_D_10		=> GPIO0_D_10,
		GPIO0_D_11		=> GPIO0_D_11,
		GPIO0_D_12		=> GPIO0_D_12,
		GPIO0_D_13		=> GPIO0_D_13,
		GPIO0_D_14		=> gpio0_clkout_0_nano,
		GPIO0_D_15		=> GPIO0_D_14,
		GPIO0_D_16		=> gpio0_clkout_1_nano,
		GPIO0_D_17		=> GPIO0_D_15,
		GPIO0_D_18		=> GPIO0_D_16,
		GPIO0_D_19		=> GPIO0_D_17,
		GPIO0_D_20		=> GPIO0_D_18,
		GPIO0_D_21		=> GPIO0_D_19,
		GPIO0_D_22		=> GPIO0_D_20,
		GPIO0_D_23		=> GPIO0_D_21,
		GPIO0_D_24		=> GPIO0_D_22,
		GPIO0_D_25		=> GPIO0_D_23,
		GPIO0_D_26		=> GPIO0_D_24,
		GPIO0_D_27		=> GPIO0_D_25,
		GPIO0_D_28		=> GPIO0_D_26,
		GPIO0_D_29		=> GPIO0_D_27,
		GPIO0_D_30		=> GPIO0_D_28,
		GPIO0_D_31		=> GPIO0_D_29,
		GPIO0_D_32		=> sd_clk_nano,
		GPIO0_D_33		=> GPIO0_D_31,
		GPIO0_CLKIN_0	=> gpio0_clkin_0_nano,
		GPIO0_CLKIN_1	=> GPIO0_CLKIN_1,
		GPIO1_D_00		=> GPIO1_D_00,
		GPIO1_D_01		=> GPIO1_D_01,
		GPIO1_D_02		=> GPIO1_D_02,
		GPIO1_D_03		=> GPIO1_D_03,
		GPIO1_D_04		=> GPIO1_D_04,
		GPIO1_D_05		=> GPIO1_D_05,
		GPIO1_D_06		=> GPIO1_D_06,
		GPIO1_D_07		=> GPIO1_D_07,
		GPIO1_D_08		=> GPIO1_D_08,
		GPIO1_D_09		=> GPIO1_D_09,
		GPIO1_D_10		=> GPIO1_D_10,
		GPIO1_D_11		=> GPIO1_D_11,
		GPIO1_D_12		=> GPIO1_D_12,
		GPIO1_D_13		=> GPIO1_D_13,
		GPIO1_D_14		=> gpio1_clkout_0_nano,
		GPIO1_D_15		=> GPIO1_D_14,
		GPIO1_D_16		=> gpio1_clkout_1_nano,
		GPIO1_D_17		=> GPIO1_D_15,
		GPIO1_D_18		=> GPIO1_D_16,
		GPIO1_D_19		=> GPIO1_D_17,
		GPIO1_D_20		=> GPIO1_D_18,
		GPIO1_D_21		=> GPIO1_D_19,
		GPIO1_D_22		=> GPIO1_D_20,
		GPIO1_D_23		=> GPIO1_D_21,
		GPIO1_D_24		=> GPIO1_D_22,
		GPIO1_D_25		=> GPIO1_D_23,
		GPIO1_D_26		=> GPIO1_D_24,
		GPIO1_D_27		=> GPIO1_D_25,
		GPIO1_D_28		=> GPIO1_D_26,
		GPIO1_D_29		=> GPIO1_D_27,
		GPIO1_D_30		=> SD_CMD,
		GPIO1_D_31		=> GPIO1_D_29,
		GPIO1_D_32		=> SD_DAT(3),
		GPIO1_D_33		=> GPIO1_D_31,
		GPIO1_CLKIN_0	=> sd_wp_nano,
		GPIO1_CLKIN_1	=> SD_DAT(0),
		GPIO2_D_00		=> gpio2_d_nano(0),
		GPIO2_D_01		=> gpio2_d_nano(1),
		GPIO2_D_02		=> gpio2_d_nano(2),
		GPIO2_D_03		=> gpio2_d_nano(3),
		GPIO2_D_04		=> gpio2_d_nano(4),
		GPIO2_D_05		=> gpio2_d_nano(5),
		GPIO2_D_06		=> gpio2_d_nano(6),
		GPIO2_D_07		=> gpio2_d_nano(7),
		GPIO2_D_08		=> gpio2_d_nano(8),
		GPIO2_D_09		=> gpio2_d_nano(9),
		GPIO2_D_10		=> gpio2_d_nano(10),
		GPIO2_D_11		=> gpio2_d_nano(11),
		GPIO2_D_12		=> gpio2_d_nano(12),
		GPIO2_CLKIN_0	=> '0',
		GPIO2_CLKIN_1	=> '0',
		GPIO2_CLKIN_2	=> '0'
	);

	gpio0_clkin_0_nano  <= GPIO0_CLKIN_0 when (SW(9) = '0') else '1';

	GPIO0_CLKOUT_0		<= gpio0_clkout_0_nano;
	GPIO0_CLKOUT_1		<= gpio0_clkout_1_nano;
	GPIO1_CLKOUT_0		<= gpio1_clkout_0_nano;
	GPIO1_CLKOUT_1		<= gpio1_clkout_1_nano;

	SD_CLK				<= sd_clk_nano;
	SD_DAT(2 downto 1)	<= (others => 'Z');

	sd_wp_nano			<= SD_WP_N;

	DRAM_BA_1			<= dram_ba_nano(1);
	DRAM_BA_0			<= dram_ba_nano(0);

	LEDG(9 downto 8)	<= "00";
	LEDG(7 downto 0)	<= led_nano;

	button_nano(1)		<= BUTTON(1);
	button_nano(0)		<= BUTTON(2) and BUTTON(0);
	sw_nano				<= not SW(3 downto 0);


-- unused pin
	FLASH_A			<= (others => '0');
	FLASH_WE_N		<= '1';
	FLASH_RESET_N	<= '0';
	FLASH_OE_N		<= '1';
	FLASH_CE_N		<= '1';
	FLASH_WP_N		<= '1';
	FLASH_BYTE_N	<= '1';
	LCD_BLON		<= '0';
	LCD_RW			<= '0';
	LCD_EN			<= '0';
	LCD_RS			<= '0';

	FLASH_D15_AM1	<= 'Z';
	FLASH_D			<= (others => 'Z');
	LCD_D			<= (others => 'Z');

	VGA_R			<= (others => '0');
	VGA_G			<= (others => '0');
	VGA_B			<= (others => '0');
	VGA_HS			<= '0';
	VGA_VS			<= '0';

	HEX3_D			<= (others => '1');
	HEX3_DP			<= '1';
	HEX2_D			<= (others => '1');
	HEX2_DP			<= '1';
	HEX1_D			<= (others => '1');
	HEX1_DP			<= '1';
	HEX0_D			<= (others => '1');
	HEX0_DP			<= '1';

	UART_TXD		<= '0';
	UART_CTS		<= '0';

	PS2_KBDAT		<= 'Z';
	PS2_KBCLK		<= 'Z';
	PS2_MSDAT		<= 'Z';
	PS2_MSCLK		<= 'Z';

	i2c_sdat_nano	<= 'Z';
	gpio2_d_nano	<= (others => 'Z');

end RTL;
