--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity SDREADIF is
	port (
		SD_DO		: in  std_logic;						-- from SD card
		SD_EJECT	: in  std_logic;						-- from SD card
		ADD			: in  std_logic_vector(15 downto 0);	-- from CPU
		DATAI		: in  std_logic_vector(7 downto 0);		-- from CPU
		WRN			: in  std_logic;						-- from CPU
		RDN			: in  std_logic;						-- from CPU
		P6_ADD		: in  std_logic_vector(15 downto 0);	-- from P6
		P6_DATAI	: in  std_logic_vector(7 downto 0);		-- from P6
		P6_M1N		: in  std_logic;						-- from P6
		P6_IORQN	: in  std_logic;						-- from P6
		P6_MREQN	: in  std_logic;						-- from P6
		P6_WRN		: in  std_logic;						-- from P6
		P6_RDN		: in  std_logic;						-- from P6
		CG0EMPTY	: in  std_logic;
		CG1EMPTY	: in  std_logic;
		PCM0EMPTY	: in  std_logic;
		PCM1EMPTY	: in  std_logic;
		CLK50M		: in  std_logic;
		RSTN		: in  std_logic;
		DATAO		: out std_logic_vector(7 downto 0);		-- to CPU
		BUFADD		: out std_logic_vector(12 downto 0);
		BUFDATAO	: out std_logic_vector(7 downto 0);
		CG0ENB		: out std_logic;
		CG0WR		: out std_logic;
		CG1ENB		: out std_logic;
		CG1WR		: out std_logic;
		PCM0ENB		: out std_logic;
		PCM0WR		: out std_logic;
		PCM1ENB		: out std_logic;
		PCM1WR		: out std_logic;
		SD_ERR		: out std_logic;
		SD_OUTENB	: out std_logic;
		SD_DI		: out std_logic;						-- to SD card
		SD_CS		: out std_logic;						-- to SD card
		SD_CLK		: out std_logic;						-- to SD card
		CGFRAME		: out std_logic_vector(3 downto 0);
		PCMFRAME	: out std_logic_vector(2 downto 0);
		CGFILEEND	: out std_logic;
		PCMFILEEND	: out std_logic;
		CGSEL		: out std_logic;
		P6_DO		: out std_logic_vector(7 downto 0);		-- to P6
		P6_CSN		: out std_logic;						-- to P6
		CTRLRST		: out std_logic;
		DBGTRG		: out std_logic_vector(7 downto 0);
		MONOUT		: out std_logic_vector(8 downto 0)
	);
end SDREADIF;

architecture RTL of SDREADIF is

	component SDREAD_CMDCNT is
		port (
			SD_DO		: in  std_logic;
			CMD_START	: in  std_logic;
			CMD_NO		: in  std_logic_vector(5 downto 0);
			SDADD		: in  std_logic_vector(31 downto 0);
			SDCNTENB	: in  std_logic;
			OUTDT		: in  std_logic_vector(7 downto 0);
			CLK			: in  std_logic;
			RSTN		: in  std_logic;
			STATEOUT	: out std_logic_vector(7 downto 0);
			INDT		: out std_logic_vector(7 downto 0);
			INDTENB		: out std_logic;
			OUTDTENB	: out std_logic;
			INDTLT2		: out std_logic;
			CMD_END		: out std_logic;
			IDLEDET		: out std_logic;
			ERRDET		: out std_logic;
			SD_DI		: out std_logic;
			SD_CS		: out std_logic;
			SD_CLK		: out std_logic
		);
	end component;

	component SPRAM_512W8B is
		port (
			address	: in  std_logic_vector(8 downto 0);
			data	: in  std_logic_vector(7 downto 0);
			wren	: in  std_logic;
			clock	: in  std_logic;
			aclr	: in  std_logic;
			q		: out std_logic_vector(7 downto 0)
		);
	end component;

	signal sdadd_i		: std_logic_vector(31 downto 0);
	signal indt_i		: std_logic_vector(7 downto 0);
	signal indtenb_i	: std_logic;
	signal outdtenb_i	: std_logic;
	signal indtlt2_i	: std_logic;
	signal cmd_end_i	: std_logic;
	signal idledet_i	: std_logic;
	signal errdet_i		: std_logic;
	signal cmd_end_f1	: std_logic;
	signal cmd_end_f2	: std_logic;
	signal sdrdenb		: std_logic;

	signal sd_rdcnt		: std_logic_vector(8 downto 0);

	signal aclr		: std_logic;
	signal buf_add	: std_logic_vector(8 downto 0);
	signal buf_dat	: std_logic_vector(7 downto 0);
	signal buf_wr	: std_logic;
	signal buf_q	: std_logic_vector(7 downto 0);

	signal p6wr			: std_logic;
	signal p6wr_f1		: std_logic;
	signal p6wr_f2		: std_logic;
	signal p6wr_f		: std_logic;

	signal p6bufwr		: std_logic;
	signal p6bufwr_f1	: std_logic;
	signal p6bufwr_f2	: std_logic;
	signal p6bufwr_r	: std_logic;

	signal reg_start		: std_logic;						-- 0x8200(0)
	signal reg_startend		: std_logic;						-- 0x8200(0)
	signal reg_idledet		: std_logic;						-- 0x8200(1)
	signal reg_errdet		: std_logic;						-- 0x8200(2)
	signal reg_inuse		: std_logic;						-- 0x8201(0)
	signal reg_sderr		: std_logic;						-- 0x8201(1)
	signal reg_cmdno		: std_logic_vector(5 downto 0);		-- 0x8205(5:0)
	signal reg_sdadd		: std_logic_vector(31 downto 0);	-- 0x8206-0x8209
	signal reg_dbgtrg		: std_logic_vector(7 downto 0);		-- 0x820F(7:0)
	signal reg_ctrlrst		: std_logic;						-- 0x8210(0)
	signal reg_bufsel		: std_logic_vector(3 downto 0);		-- 0x8211(3:0)
	signal reg_sdrdinit		: std_logic;						-- 0x8213(0)
	signal reg_sdrdlen		: std_logic;						-- 0x8213(1)
	signal reg_sdrdfull		: std_logic;						-- 0x8213(4)
	signal reg_bufupper		: std_logic_vector(3 downto 0);		-- 0x8214(3:0)
	signal reg_cgframe		: std_logic_vector(3 downto 0);		-- 0x8216(3:0)
	signal reg_pcmframe		: std_logic_vector(2 downto 0);		-- 0x8217(2:0)
	signal reg_cgfileend	: std_logic;						-- 0x8218(0)
	signal reg_pcmfileend	: std_logic;						-- 0x8219(0)
	signal reg_cmdend		: std_logic;						-- 0x821C(0)/0x14(0)

	signal reg_command		: std_logic_vector(7 downto 0);		-- 0x12(7:0)/0x821A(7:0)
	signal reg_cgsel		: std_logic;						-- 0x15(0)
	signal reg_cmdstart		: std_logic;						-- 0x13(0)/0x821B(0)

	signal p6_do_i		: std_logic_vector(7 downto 0);
	signal p6_csn_i		: std_logic;

	signal reg_start_f1	: std_logic;
	signal reg_start_f2	: std_logic;
	signal reg_start_r	: std_logic;

	signal reg_startlt1	: std_logic;
	signal reg_startlt2	: std_logic;

	signal datao_i		: std_logic_vector(7 downto 0);

	signal rdn_f1	: std_logic;
	signal rdn_f2	: std_logic;
	signal wrn_f1	: std_logic;
	signal wrn_f2	: std_logic;
	signal add_f1	: std_logic_vector(15 downto 0);
	signal datai_f1	: std_logic_vector(7 downto 0);
	signal rdp		: std_logic;
	signal wrp		: std_logic;

	signal statecnt	: std_logic_vector(7 downto 0);

begin

	U_SDREAD_CMDCNT : SDREAD_CMDCNT
	port map (
		SD_DO		=> SD_DO,
		CMD_START	=> reg_startlt2,
		CMD_NO		=> reg_cmdno,
		SDADD		=> sdadd_i,
		SDCNTENB	=> reg_inuse,
		OUTDT		=> (others => '0'),
		CLK			=> CLK50M,
		RSTN		=> RSTN,
		STATEOUT	=> statecnt,
		INDT		=> indt_i,
		INDTENB		=> indtenb_i,
		OUTDTENB	=> outdtenb_i,
		INDTLT2		=> indtlt2_i,
		CMD_END		=> cmd_end_i,
		IDLEDET		=> idledet_i,
		ERRDET		=> errdet_i,
		SD_DI		=> SD_DI,
		SD_CS		=> SD_CS,
		SD_CLK		=> SD_CLK
	);

	sdadd_i <= reg_sdadd(22 downto 0) & "000000000";

-- SDcard buffer
	U_SPRAM_512W8B : SPRAM_512W8B
	port map (
		address	=> buf_add,
		data	=> buf_dat,
		wren	=> buf_wr,
		clock	=> CLK50M,
		aclr	=> aclr,
		q		=> buf_q
	);

	aclr     <= not RSTN;

	buf_add  <=	P6_ADD(0) & P6_ADD(15 downto 8) when (reg_bufsel = "1010") else
				add_f1(8 downto 0)              when (reg_bufsel = "1001") else
				sd_rdcnt;

	buf_dat  <= P6_DATAI when (reg_bufsel = "1010") else
				indt_i;

	buf_wr   <=	p6bufwr_r when (reg_bufsel = "1010") else
				sdrdenb   when (reg_bufsel = "1000") else
				'0';


	p6wr <= '1' when (P6_IORQN = '0' and P6_WRN = '0') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			p6wr_f1 <= '0';
			p6wr_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			p6wr_f1 <= p6wr;
			p6wr_f2 <= p6wr_f1;
		end if;
	end process;

	p6wr_f <= '1' when (p6wr_f1 = '0' and p6wr_f2 = '1') else '0';


	p6bufwr <=	'1' when (p6wr = '1' and P6_ADD(7 downto 0) = X"10") else
				'1' when (p6wr = '1' and P6_ADD(7 downto 0) = X"11") else
				'0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			p6bufwr_f1 <= '0';
			p6bufwr_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			p6bufwr_f1 <= p6bufwr;
			p6bufwr_f2 <= p6bufwr_f1;
		end if;
	end process;

	p6bufwr_r <= '1' when (p6bufwr_f1 = '1' and p6bufwr_f2 = '0') else '0';



-- CPU I/F
-- data latch
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			rdn_f1   <= '1';
			rdn_f2   <= '1';
			wrn_f1   <= '1';
			wrn_f2   <= '1';
			add_f1   <= (others => '0');
			datai_f1 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			rdn_f1   <= RDN;
			rdn_f2   <= rdn_f1;
			wrn_f1   <= WRN;
			wrn_f2   <= wrn_f1;
			add_f1   <= ADD;
			datai_f1 <= DATAI;
		end if;
	end process;

	rdp <= '1' when (rdn_f1 = '0' and rdn_f2 = '1') else '0';
	wrp <= '1' when (wrn_f1 = '0' and wrn_f2 = '1') else '0';

-- register
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_start     <= '0';
			reg_inuse     <= '0';
			reg_sderr     <= '0';
			reg_cmdno     <= (others => '0');
			reg_sdadd     <= (others => '0');
			reg_dbgtrg    <= (others => '0');
			reg_ctrlrst   <= '0';
			reg_bufsel    <= (others => '0');
			reg_sdrdinit  <= '0';
			reg_sdrdlen   <= '0';
			reg_bufupper  <= (others => '0');
			reg_cgframe   <= (others => '0');
			reg_pcmframe  <= (others => '0');
			reg_cgfileend <= '0';
			reg_pcmfileend<= '0';
			reg_cmdend    <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (wrp = '1') then
				case add_f1 is
					when X"8200" => reg_start                  <= datai_f1(0);
					when X"8201" =>	reg_inuse                  <= datai_f1(0);
									reg_sderr                  <= datai_f1(1);
					when X"8205" => reg_cmdno                  <= datai_f1(5 downto 0);
					when X"8206" => reg_sdadd( 7 downto  0)    <= datai_f1;
					when X"8207" => reg_sdadd(15 downto  8)    <= datai_f1;
					when X"8208" => reg_sdadd(23 downto 16)    <= datai_f1;
					when X"8209" => reg_sdadd(31 downto 24)    <= datai_f1;
					when X"820F" => reg_dbgtrg                 <= datai_f1;
					when X"8210" => reg_ctrlrst                <= datai_f1(0);
					when X"8211" => reg_bufsel                 <= datai_f1(3 downto 0);
					when X"8213" => reg_sdrdinit               <= datai_f1(0);
									reg_sdrdlen                <= datai_f1(1);
					when X"8214" => reg_bufupper               <= datai_f1(3 downto 0);
					when X"8216" => reg_cgframe                <= datai_f1(3 downto 0);
					when X"8217" => reg_pcmframe               <= datai_f1(2 downto 0);
					when X"8218" => reg_cgfileend              <= datai_f1(0);
					when X"8219" => reg_pcmfileend             <= datai_f1(0);
					when X"821C" => reg_cmdend                 <= datai_f1(0);
					when others  => null;
				end case;
			end if;
		end if;
	end process;

-- start_end register
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmd_end_f1 <= '0';
			cmd_end_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			cmd_end_f1 <= cmd_end_i;
			cmd_end_f2 <= cmd_end_f1;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_start_f1 <= '0';
			reg_start_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			reg_start_f1 <= reg_start;
			reg_start_f2 <= reg_start_f1;
		end if;
	end process;

	reg_start_r <= '1' when (reg_start_f1 = '1' and reg_start_f2 = '0') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_startlt1 <= '0';
			reg_startlt2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_start_r = '1') then
				reg_startlt1 <= '1';
			elsif (indtlt2_i = '1') then
				reg_startlt1 <= '0';
			end if;

			if (indtlt2_i = '1') then
				reg_startlt2 <= reg_startlt1;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_startend <= '0';
			reg_idledet  <= '0';
			reg_errdet   <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (cmd_end_f1 = '1' and cmd_end_f2 = '0') then
				reg_startend <= '0';
			elsif (reg_start_r = '1') then
				reg_startend <= '1';
			end if;
			if (cmd_end_f1 = '1' and cmd_end_f2 = '0') then
				reg_idledet  <= idledet_i;
				reg_errdet   <= errdet_i;
			end if;
		end if;
	end process;

-- register read
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			datao_i <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (rdp = '1') then
				case add_f1 is
					when X"8200" => datao_i <= "00000" & reg_errdet & reg_idledet & reg_startend;
					when X"8201" =>	datao_i <= "000000" & reg_sderr & reg_inuse;
					when X"8204" => datao_i <= "0000000" & SD_EJECT;
					when X"8205" => datao_i <= "00" & reg_cmdno;
					when X"8206" => datao_i <= reg_sdadd( 7 downto  0);
					when X"8207" => datao_i <= reg_sdadd(15 downto  8);
					when X"8208" => datao_i <= reg_sdadd(23 downto 16);
					when X"8209" => datao_i <= reg_sdadd(31 downto 24);
					when X"820F" => datao_i <= reg_dbgtrg;
					when X"8210" => datao_i <= "0000000" & reg_ctrlrst;
					when X"8211" => datao_i <= "0000" & reg_bufsel;
					when X"8213" => datao_i <= "000" & reg_sdrdfull & "00" & reg_sdrdlen & reg_sdrdinit;
					when X"8214" => datao_i <= "0000" & reg_bufupper;
					when X"8215" => datao_i <= "0000" & PCM1EMPTY & PCM0EMPTY & CG1EMPTY & CG0EMPTY;
					when X"8216" => datao_i <= "0000" & reg_cgframe;
					when X"8217" => datao_i <= "00000" & reg_pcmframe;
					when X"8218" => datao_i <= "0000000" & reg_cgfileend;
					when X"8219" => datao_i <= "0000000" & reg_pcmfileend;
					when X"821A" => datao_i <= reg_command;
					when X"821B" => datao_i <= "0000000" & reg_cmdstart;
					when X"821D" => datao_i <= "0000000" & reg_cgsel;
					when others  =>
						if (add_f1(15 downto 9) = "1000000") then
							datao_i <= buf_q;
						else
							datao_i <= X"FF";
						end if;
				end case;
			end if;
		end if;
	end process;


-- SD read data
	sdrdenb <= '1' when (indtenb_i = '1' and indtlt2_i = '1') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			sd_rdcnt <= (others => '1');
			reg_sdrdfull <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_sdrdinit = '1') then
				sd_rdcnt <= (others => '0');
				reg_sdrdfull <= '0';
			elsif (sdrdenb = '1') then
				if (sd_rdcnt = "111111111") then
					reg_sdrdfull <= '1';
				else
					sd_rdcnt <= sd_rdcnt + 1;
				end if;
			end if;
		end if;
	end process;


-- for P6 port
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_command  <= (others => '0');
			reg_cmdstart <= '0';
			reg_cgsel    <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (p6wr_f = '1') then
				if (P6_ADD(7 downto 0) = X"12") then
					reg_command  <= P6_DATAI;
				end if;
				if (P6_ADD(7 downto 0) = X"13") then
					reg_cmdstart <= P6_DATAI(0);
				end if;
				if (P6_ADD(7 downto 0) = X"15") then
					reg_cgsel    <= P6_DATAI(0);
				end if;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			p6_do_i  <= (others => '1');
			p6_csn_i <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (P6_IORQN = '0' and P6_RDN = '0') then
				if    (P6_ADD(7 downto 0) = X"10") then
					p6_do_i  <= buf_q;
					p6_csn_i <= '0';
				elsif (P6_ADD(7 downto 0) = X"11") then
					p6_do_i  <= buf_q;
					p6_csn_i <= '0';
				elsif (P6_ADD(7 downto 0) = X"14") then
					p6_do_i  <= "1111111" & reg_cmdend;
					p6_csn_i <= '0';
				else
					p6_do_i  <= (others => '1');
					p6_csn_i <= '1';
				end if;
			else
				p6_do_i  <= (others => '1');
				p6_csn_i <= '1';
			end if;
		end if;
	end process;


	DATAO     <= datao_i;

	SD_ERR    <= reg_sderr;
	SD_OUTENB <= reg_inuse;

	BUFADD    <= reg_bufupper & sd_rdcnt;
	BUFDATAO  <= indt_i;

	CG0ENB    <= '1' when (reg_bufsel = "0100") else '0';
	CG1ENB    <= '1' when (reg_bufsel = "0101") else '0';
	PCM0ENB   <= '1' when (reg_bufsel = "0110") else '0';
	PCM1ENB   <= '1' when (reg_bufsel = "0111") else '0';
	CG0WR     <= '1' when (reg_bufsel = "0100" and sdrdenb = '1') else '0';
	CG1WR     <= '1' when (reg_bufsel = "0101" and sdrdenb = '1') else '0';
	PCM0WR    <= '1' when (reg_bufsel = "0110" and sdrdenb = '1') else '0';
	PCM1WR    <= '1' when (reg_bufsel = "0111" and sdrdenb = '1') else '0';

	CGFRAME   <= reg_cgframe;
	PCMFRAME  <= reg_pcmframe;
	CGFILEEND <= reg_cgfileend;
	PCMFILEEND<= reg_pcmfileend;
	CGSEL     <= reg_cgsel;

	P6_DO     <= p6_do_i;
	P6_CSN    <= p6_csn_i;

	CTRLRST   <= reg_ctrlrst;

	DBGTRG    <= reg_dbgtrg;

--	MONOUT(3 downto 0) <= reg_bufsel;
--	MONOUT(4) <= reg_cmdstart;
--	MONOUT(5) <= reg_cmdend;
--	MONOUT(6) <= buf_wr;
--	MONOUT(7) <= p6_csn_i;
	MONOUT(8) <= '0';

	MONOUT(5 downto 0) <= reg_cmdno;
	MONOUT(6) <= reg_startlt2;
	MONOUT(7) <= errdet_i;

end RTL;
